# -*- coding: utf-8 -*-
"""
***************************************************************************
    GIS LMEC Bologna, PyQGIS Network Analysis basics
    ---------------------
    
    Date                 : March 2019
    Author...            : Alexander Lehner
***************************************************************************
*                                                                         *
*   This file does the most basic steps to calculate the distance/time    *
*   within a given (handdrawn) network, utilising elements from PyQGIS    *
*   Dijkstra's algorithm is used                                          *
*   Everything adapted and rewritten for PqGQIS3/Python3                  *
*                                                                         *
***************************************************************************
"""



# from qgis.networkanalysis import *


pathpoint = '/path/to/your/file/Bairoch_v2.gpkg|layername=Bairoch_v2'
points_layer = points_layer = QgsVectorLayer(pathpoint, 'points', "ogr")


# could load a layer from the interface:
#layer = qgis.utils.iface.mapCanvas().currentLayer()


pathroad = '/path/to/your/file//GIS_data/GIS_LMEC.gpkg|layername=Road'
roads_layer = QgsVectorLayer(pathroad, 'roads', "ogr")


# first try by hand longlat
#########################################
myCRS = points_layer.sourceCrs()
# we need a director
graphDirector = QgsVectorLayerDirector(roads_layer, -1, '', '', '', QgsVectorLayerDirector.DirectionBoth)

# then we need a properter ("strategy")
speedFieldIndex = roads_layer.fields().lookupField('cost') # could have also used the indexFromName() method | or the .dataProvider().fieldNameIndex(attributeName) method
strategy = QgsNetworkSpeedStrategy(speedFieldIndex, 50, 1000) # number of cost field, defaultSpeed, speedToDistanceFactor
#strategy = QgsNetworkDistanceStrategy() #if we want to go for distances ("shortest"), does not need any arguments
graphDirector.addStrategy(strategy)

# define points
points_to_tie = [QgsPointXY(11.32959511152520804, 44.4622508930552911), QgsPointXY(11.30402470490417954, 44.24367399482645169)]

# then we will create the graph (with the QgsGraphBuilder class constructor)
builder = QgsGraphBuilder(myCRS, topologyTolerance = 0)

# tie the points to the graph
tiedPoints = graphDirector.makeGraph(builder, points_to_tie)

# build the graph
graph = builder.graph()
# let's check a bit:
graph.vertexCount()
graph.edgeCount()

# now we need to tell the algorithm on what exact vertex to start and end
startId = graph.findVertex(tiedPoints[0])
endId   = graph.findVertex(tiedPoints[1])

(dtree, dcost) = QgsGraphAnalyzer.dijkstra(graph, startId, 0)
dcost[endId]

print("Travel time is", round(dcost[endId], 3), "hours.")


