# -*- coding: utf-8 -*-
"""
***************************************************************************
    GIS LMEC Bologna, PyQGIS Network Analysis: first automatisation
    ---------------------
    
    Date                 : March 2019
    Author...            : Alexander Lehner
***************************************************************************
*                                                                         *
*   This file does the most basic steps to calculate the distance/time    *
*   within a given (handdrawn) network, utilising elements from PyQGIS    *
*   A first simple for-loop is used in order to illustrate                *
*   how to iterate over many points                                       *
*   Dijkstra's algorithm is used                                          *
*   Everything adapted and rewritten for PqGQIS3/Python3                  *
*                                                                         *
***************************************************************************
"""



# same procedure as before, part1
pathpoint = '/path/to/your/file/Bairoch_v2.gpkg|layername=Bairoch_v2'
points_layer = points_layer = QgsVectorLayer(pathpoint, 'points', "ogr")

pathroad = '/path/to/your/file/GIS_LMEC.gpkg|layername=Road'
#pathroad = '/path/to/your/file//GIS_data/Roman Road Network (version 2008)/RomanRoads_EPSG4326.gpkg|layername=RomanRoads_EPSG4326'

# now we loop over things
#########################################

# a testloop
for i in range(10):
    print(i)

# let's check what's inside:
for f in points_layer.getFeatures():
    f['citybb']
    

for f in points_layer.getFeatures():
    f[2]
    
    
# the only "moving things" are the start and the end points
# so we retrieve the geometry from the actual layer
features = points_layer.getFeatures()
for feature in features:
    geom = feature.geometry()
    #geom.asPoint()
    feature.geometry().asPoint()
    


# before we loop, we need to create a container to store our results, iteration by iteration
results = [] # for simplicity let's just create an empty list (n.b. for bigger tasks it is not good practice to expand the size of the container with each iteration)

# for now we choose just one starting point
startcity = 'Bologna'
startXY = QgsPointXY(11.3341, 44.4997)

# now we start to loop
# n.b. we need to build the graph in every iteration with the new point(s)!

for f in points_layer.getFeatures():
    # 1. load the network
    roads_layer = QgsVectorLayer(pathroad, 'roads', "ogr")
    
    # 2. we do exactly the same as before but INDENT with one tab
    
    myCRS = points_layer.sourceCrs()
    # we need a director
    graphDirector = QgsVectorLayerDirector(roads_layer, -1, '', '', '', QgsVectorLayerDirector.DirectionBoth)   
    # then we need a properter ("strategy")
    speedFieldIndex = roads_layer.fields().lookupField('cost') # could have also used the indexFromName() method | or the .dataProvider().fieldNameIndex(attributeName) method
    strategy = QgsNetworkSpeedStrategy(speedFieldIndex, 50, 1000) # number of cost field, defaultSpeed, speedToDistanceFactor
    #strategy = QgsNetworkDistanceStrategy() #if we want to go for distances ("shortest"), does not need any arguments
    graphDirector.addStrategy(strategy)
    
    # HERE WE NEED TO MAKE A CHANGE 
    # define points
    points_to_tie = [startXY, f.geometry().asPoint()] 
    # then we will create the graph (with the QgsGraphBuilder class constructor)
    builder = QgsGraphBuilder(myCRS, topologyTolerance = 0) 
    # tie the points to the graph
    tiedPoints = graphDirector.makeGraph(builder, points_to_tie)    
    # build the graph
    graph = builder.graph() 
    # now we need to tell the algorithm on what exact vertex to start and end
    startId = graph.findVertex(tiedPoints[0])
    endId   = graph.findVertex(tiedPoints[1])   
    (dtree, dcost) = QgsGraphAnalyzer.dijkstra(graph, startId, 0)
    #dcost[endId]
    
    # now we append our results to the container above, iteration by iteration
    results.append([startcity, f['citybb'], round(dcost[endId], 3)])
    
    # before the next iteration we clear the storage
    del graph 
    del tiedPoints
    del points_to_tie
    del startId
    del endId
    del dtree
    del dcost
